//Console input parser unit
//  coninp_p.pas
//  (c) by Dust Signs Andreas Unterweger 2006

unit coninp_p;

interface

uses
  coninp_h;

type
  TCustomConsoleInputParser = class
    protected
      FInput: String;
      FCommand: String;
      FParameters: TStringArray;
      procedure FParse; virtual; abstract;
    public
      constructor Create(AnInput: String = '');
      procedure SetInput(AnInput: String);
      function GetCommand: String;
      function GetParameters: TStringArray;
    end;

  TSimpleConsoleInputParser = class(TCustomConsoleInputParser)
    protected
      procedure FParse; override;
    end;

  TAdvancedConsoleInputParser = class(TSimpleConsoleInputParser)
    protected
      procedure FSummarizeQuotedParameters(AQuoteChar: Char);
      procedure FParse; override;
    end;

implementation

constructor TCustomConsoleInputParser.Create(AnInput: String);
begin
  SetInput(AnInput);
  FParse;
end;

procedure TCustomConsoleInputParser.SetInput(AnInput: String);
begin
  FInput := AnInput;
  FParse;
end;

function TCustomConsoleInputParser.GetCommand: String;
begin
  Result := FCommand;
end;

function TCustomConsoleInputParser.GetParameters: TStringArray;
begin
  Result := FParameters;
end;

procedure TSimpleConsoleInputParser.FParse;
var
  temp: String;
begin
  SetLength(FParameters, 0);
  if Pos(' ', FInput) = 0 then begin //No parameters
    FCommand := FInput;
    exit;
    end;
  FCommand := Copy(FInput, 1, Pos(' ', FInput) - 1);
  temp := FInput;
  Delete(temp, 1, Pos(' ', temp));
  while Pos(' ', temp) > 0 do begin
    SetLength(FParameters, Length(FParameters) + 1);
    FParameters[High(FParameters)] := Copy(temp, 1, Pos(' ', temp) - 1);
    Delete(temp, 1, Pos(' ', temp));
    end;
  SetLength(FParameters, Length(FParameters) + 1); //Add last parameter...
  FParameters[High(FParameters)] := temp; //...which is the rest string stored in temp
end;

procedure TAdvancedConsoleInputParser.FSummarizeQuotedParameters(AQuoteChar: Char);
var
  i, j, k: Integer;
begin
  i := 0;
  while i <= High(FParameters) do begin
    if Pos(AQuoteChar, FParameters[i]) = 1 then begin //Is there an " at the beginning of [i]?
      for j := i + 1 to High(FParameters) do begin //Continue search for "
        if Pos(AQuoteChar, FParameters[j]) = Length(FParameters[j]) then begin //Is there an " at the end of [j]?
          for k := i + 1 to j do //Copy everything inbetween into [i]
            FParameters[i] := FParameters[i] + ' ' + FParameters[k];
          for k := i + 1 to High(FParameters) - (j - i) do //Move other FParameters backwards and overwrite unneeded FParameters between [i + 1] and [j]
            FParameters[k] := FParameters[k + (j - i)];
          SetLength(FParameters, Length(FParameters) - (j - i)); //Reduce number of FParameters
          break; //Quit inner loop
          end;
        end;
      end;
    Inc(i);
    end;
  for i := 0 to High(FParameters) do begin //Remove quote characters
    if Length(FParameters[i]) > 2 then begin
      if (FParameters[i][1] = AQuoteChar) and (FParameters[i][Length(FParameters[i])] = AQuoteChar) then //Do first and last char match?
        FParameters[i] := Copy(FParameters[i], 2, Length(FParameters[i]) - 2) //Copy and leave out first and last char (in total 2)
      end;
    if (FParameters[i] = AQuoteChar) or (FParameters[i] = AQuoteChar + AQuoteChar) then
      FParameters[i] := ''; //Make parameter empty if constists only of quote characters
    end;
end;

procedure TAdvancedConsoleInputParser.FParse;
var
  i, j: Integer;
begin
  inherited FParse;
  FSummarizeQuotedParameters('"'); //"
  FSummarizeQuotedParameters(''''); //'
  i := 0;
  while i <= High(FParameters) do begin //Delete empty parameters
    if FParameters[i] = '' then begin
      for j := i to High(FParameters) - 1 do
        FParameters[j] := FParameters[j + 1];
      SetLength(FParameters, Length(FParameters) - 1);
      Dec(i); //Repeat procedure for next parameter (which has been moved back to [i]'s position)
      end;
    Inc(i);
    end;
end;

end.
