//Console input handler unit 2 (examples)
//  coninph2.pas
//  (c) by Dust Signs Andreas Unterweger 2006

unit coninph2;

interface

uses
  coninp_h, SysUtils;

var
  EchoCommandHandler: TCommandHandler;
  AboutCommandHandler: TCommandHandler;
  ExitCommandHandler: TCommandHandler;
  StateCommandHandler: TCommandHandler;

function Validate_About(AParameterList: TStringArray): THandlerResult;
function Validate_State(AParameterList: TStringArray): THandlerResult;

function Execute_Echo(AParameterList: TStringArray): THandlerResult;
function Execute_About(AParameterList: TStringArray): THandlerResult;
function Execute_Exit(AParameterList: TStringArray): THandlerResult;
function Execute_State(AParameterList: TStringArray): THandlerResult;

implementation

function Validate_About(AParameterList: TStringArray): THandlerResult;
begin
  InitializeHandlerResult(Result);
  if Length(AParameterList) > 0 then
    SetSpecialHandlerResult(Result, HR_TOO_MANY_PARAMS);
end;

function Execute_About(AParameterList: TStringArray): THandlerResult;
begin
  InitializeHandlerResult(Result);
  Result._Result := 'Console input handler (coninp_h) - (c) by Dust Signs Andreas Unterweger 2006';
end;

function Execute_Echo(AParameterList: TStringArray): THandlerResult;
var
  i: Integer;
begin
  InitializeHandlerResult(Result);
  for i := 0 to High(AParameterList) do
    Result._Result := Result._Result + AParameterList[i] + #13#10;
  SetLength(Result._Result, Length(Result._Result) - 2); //Remove last #13#10
end;

function Execute_Exit(AParameterList: TStringArray): THandlerResult;
begin
  InitializeHandlerResult(Result);
  SetSpecialHandlerResult(Result, HR_REQUIRES_EXIT);
end;

function Validate_State(AParameterList: TStringArray): THandlerResult;
begin
  InitializeHandlerResult(Result);
  if Length(AParameterList) = 0 then
    SetSpecialHandlerResult(Result, HR_TOO_LESS_PARAMS);
  if Length(AParameterList) > 1 then
    SetSpecialHandlerResult(Result, HR_TOO_MANY_PARAMS);
end;

function Execute_State(AParameterList: TStringArray): THandlerResult;
begin
  InitializeHandlerResult(Result);
  SetSpecialHandlerResult(Result, HR_REQUIRES_STATUS_CHANGE);
  Result.DataFormat := DF_STATUS_INFORMATION;
  Result.AdditionalData := PChar(AParameterList[0]);
end;

initialization
  EchoCommandHandler := TCommandHandler.Create('echo', nil, Execute_Echo);
  AboutCommandHandler := TCommandHandler.Create('about', Validate_About, Execute_About);
  ExitCommandHandler := TCommandHandler.Create('exit', nil, Execute_Exit);
  StateCommandHandler := TCommandHandler.Create('cs', Validate_State, Execute_State);

finalization
  FreeAndNil(EchoCommandHandler);
  FreeAndNil(AboutCommandHandler);
  FreeAndNil(ExitCommandHandler);
  FreeAndNil(StateCommandHandler);

end.
