//Console input handler unit 3 (extended examples)
//  coninph3.pas
//  (c) by Dust Signs Andreas Unterweger 2006

unit coninph3;

interface

uses
  coninp_h, SysUtils;

var
  ReverseEchoCommandHandler: TCommandHandler;
  ChangeDirectoryCommandHandler: TCommandHandler;
  ListDirectoryCommandHandler: TCommandHandler;

function Validate_ChangeDirectory(AParameterList: TStringArray): THandlerResult;
function Validate_ListDirectory(AParameterList: TStringArray): THandlerResult;

function Execute_ReverseEcho(AParameterList: TStringArray): THandlerResult;
function Execute_ChangeDirectory(AParameterList: TStringArray): THandlerResult;
function Execute_ListDirectory(AParameterList: TStringArray): THandlerResult;

implementation

var
  TempPath: String; //Store path information of "cd" (storing it in a PChar does not work!?)

function Execute_ReverseEcho(AParameterList: TStringArray): THandlerResult;
var
  i: Integer;
begin
  InitializeHandlerResult(Result);
  for i := High(AParameterList) downto 0 do //Reverse order
    Result._Result := Result._Result + AParameterList[i] + #13#10;
  SetLength(Result._Result, Length(Result._Result) - 2); //Remove last #13#10
end;

function Validate_ChangeDirectory(AParameterList: TStringArray): THandlerResult;
begin
  InitializeHandlerResult(Result);
  if Length(AParameterList) = 0 then
    SetSpecialHandlerResult(Result, HR_TOO_LESS_PARAMS);
  if Length(AParameterList) > 1 then
    SetSpecialHandlerResult(Result, HR_TOO_MANY_PARAMS);
end;

function Execute_ChangeDirectory(AParameterList: TStringArray): THandlerResult;
begin
  InitializeHandlerResult(Result);
  {$I-}
  ChDir(AParameterList[0]); //Change directory
  {$I+}
  if IOResult <> 0 then begin
    Result.ErrorCode := HR_PARAMS_INVALID;
    Result._Result := 'Could not change directory';
  end else begin
    SetSpecialHandlerResult(Result, HR_REQUIRES_STATUS_CHANGE);
    Result.DataFormat := DF_STATUS_INFORMATION;
    TempPath := GetCurrentDir;
    Result.AdditionalData := PChar(TempPath);
    end;
end;

function Validate_ListDirectory(AParameterList: TStringArray): THandlerResult;
begin
  InitializeHandlerResult(Result);
  if Length(AParameterList) > 1 then
    SetSpecialHandlerResult(Result, HR_TOO_MANY_PARAMS);
end;

function Execute_ListDirectory(AParameterList: TStringArray): THandlerResult;
var
  r: TSearchRec;
  search_string: String;
  FileCount: Integer;
begin
  InitializeHandlerResult(Result);
  search_string := IncludeTrailingPathDelimiter(GetCurrentDir); //=> IncludeTrailingBackslash
  if Length(AParameterList) = 1 then //If parameter specified...
    search_string := search_string + AParameterList[0] //...include in search
  else
    search_string := search_string + '*.*'; //Else list all files
  if FindFirst(search_string, faAnyFile, r) <> 0 then begin
    FindClose(r);
    Result._Result := 'No files in ' + GetCurrentDir;
    exit;
    end;
  FileCount := 0;
  repeat
    Result._Result := Result._Result + r.Name + #13#10;
    Inc(FileCount);
  until
    FindNext(r) <> 0;
  FindClose(r);
  Result._Result := Result._Result + #13#10 + IntToStr(FileCount) + ' files found.';
end;

initialization
  ReverseEchoCommandHandler := TCommandHandler.Create('recho', nil, Execute_ReverseEcho);
  ChangeDirectoryCommandHandler := TCommandHandler.Create('cd', Validate_ChangeDirectory, Execute_ChangeDirectory);
  ListDirectoryCommandHandler := TCommandHandler.Create('dir', Validate_ListDirectory, Execute_ListDirectory);

finalization
  FreeAndNil(ReverseEchoCommandHandler);
  FreeAndNil(ChangeDirectoryCommandHandler);
  FreeAndNil(ListDirectoryCommandHandler);

end.
