//Console handler unit
//  console_h.pas
//  (c) by Dust Signs Andreas Unterweger 2006

unit console_h;

interface

uses
  coninp_h, coninp_p, SysUtils;

type
  TPostExecuteReactionFunction = function(AnOutput: THandlerResult): String;

  TCustomConsoleHandler = class
    protected
      FParser: TCustomConsoleInputParser;
      FInputHandler: TConsoleInputHandler;
      FState: String;
      procedure FPre_Execute(AnInput: String); virtual; abstract;
      function FPost_Execute(AnOutput: THandlerResult): Boolean; virtual; abstract;
    public
      constructor Create(AParser: TCustomConsoleInputParser; AnInputHandler: TConsoleInputHandler; InitialState: String = '');
      function Execute(AnInput: String): Boolean;
    end;

  TPrintableConsoleHandler = class(TCustomConsoleHandler)
    protected
      FDelimiter: String;
      procedure FPre_Execute(AnInput: String); override;
      function FPost_Execute(AnOutput: THandlerResult): Boolean; override;
    public
      constructor Create(AParser: TCustomConsoleInputParser; AnInputHandler: TConsoleInputHandler; InitialState: String = '');
      procedure SetDelimiter(ADelimiter: String);
    end;

  TCustomizeablePrintableConsoleHandler = class(TPrintableConsoleHandler)
    protected
      FPostExecuteReaction: TPostExecuteReactionFunction;
      function FPost_Execute(AnOutput: THandlerResult): Boolean; override;
    public
      constructor Create(AParser: TCustomConsoleInputParser; AnInputHandler: TConsoleInputHandler; InitialState: String = ''; PostExecuteReaction: TPostExecuteReactionFunction = nil);
    end;

implementation

constructor TCustomConsoleHandler.Create(AParser: TCustomConsoleInputParser; AnInputHandler: TConsoleInputHandler; InitialState: String = '');
begin
  FParser := AParser;
  FInputHandler := AnInputHandler;
  FState := InitialState;
end;

function TCustomConsoleHandler.Execute(AnInput: String): Boolean;
var
  _result: THandlerResult;
begin
  FParser.SetInput(AnInput); //Set and parse input
  FPre_Execute(AnInput);
  _result := FInputHandler.Execute(FParser.GetCommand, FParser.GetParameters);
  Result := FPost_Execute(_result);
end;

procedure TPrintableConsoleHandler.FPre_Execute(AnInput: String);
begin
  ; //Don't do anything here (only implement to make class not abstract
end;

function TPrintableConsoleHandler.FPost_Execute(AnOutput: THandlerResult): Boolean;
begin
  Result := true;
  if AnOutput.ErrorCode <> HR_REQUIRES_EXIT then begin
    case AnOutput.ErrorCode of
      HR_NO_HANDLER_FOUND: WriteLn('Invalid command (no handler found for given command)');
      HR_REQUIRES_STATUS_CHANGE: begin
                                   if AnOutput.DataFormat = DF_STATUS_INFORMATION then
//                                     FState := AnOutput.AdditionalData;
//                                     FState := String(AnOutput.AdditionalData);
//                                     FState := Copy(AnOutput.AdditionalData, 1, Length(AnOutput.AdditionalData));
                                     FState := StrPas(AnOutput.AdditionalData);
                                 end;
      else
        WriteLn(AnOutput._Result);
      end;
  end else begin
    Result := false;
    exit; //Exit because of HR_REQUIRES_EXIT
    end;
  Write(FState, FDelimiter);
end;

constructor TPrintableConsoleHandler.Create(AParser: TCustomConsoleInputParser; AnInputHandler: TConsoleInputHandler; InitialState: String = '');
begin
  inherited;
  FDelimiter := '>';
  Write(InitialState, FDelimiter);
end;

procedure TPrintableConsoleHandler.SetDelimiter(ADelimiter: String);
begin
  FDelimiter := ADelimiter;
end;

function TCustomizeablePrintableConsoleHandler.FPost_Execute(AnOutput: THandlerResult): Boolean;
begin
  if Assigned(FPostExecuteReaction) then
    AnOutput._Result := FPostExecuteReaction(AnOutput); //Give ExecuteReactionFunction a chance to modify the output
  Result := inherited FPost_Execute(AnOutput);
end;

constructor TCustomizeablePrintableConsoleHandler.Create(AParser: TCustomConsoleInputParser; AnInputHandler: TConsoleInputHandler; InitialState: String = ''; PostExecuteReaction: TPostExecuteReactionFunction = nil);
begin
  inherited Create(AParser, AnInputHandler, InitialState);
  FPostExecuteReaction := PostExecuteReaction;
end;

end.
